!**********************************************************
! Module: rgsimplot_mod
!
! Authors:   Jon Rainger - MSSL
!          
! Description: produces rgs banana plots
! 
! Comments: 
! 
! $Id: module.f90,v 1.5 1998/01/13 08:39:15 gvacanti Exp $
!         28.9.2000 (CB) - code tidied up 
!-        27.9.2000 (CB) - Input parameter "sourceid" removed.
!
!         25.6.2000 - Modified by CB-
!                Code tidied up.
!                Graph character sizes scaled to size of graph.
!                Boolean paramater "hardCopy" removed.
!                Layout of exposure details changed.      
!         14.9.2000 - Modified (CB) - pgplot device opened after
!                     input parameters checked. Subroutines
!                     "GetString" and "GetReal" added so that
!                      the program won't crash if the attributes
!                      aren't present in the fits file. 
!         05.11.2000 Source coordinates added to graph
!                    program version no added.
!         31.01.2001 Program modified to use output from rgsregions  
!         28.03.2001 Background fits region now drawn in red.
!         25.04.2001 Changes made to enable it to work with the new
!                    version of rgssources. (SOURCES table changed
!                    to SRCLIST)
!         08.05.2001 Fixed SPR 2372
!         01.06.2001 Removed code that fixes the y-axes scales-
!                    this caused a problem in the drawing of the image-
!                    Fixes SPR 2399)
!         26.10.2005 VNY continuing fixing the SPR-3523
!************************************************************
module rgsimplot_mod
    use dal
    use cal
    use oal
    use param
    use errorHandling  
    implicit none 

!***************************************
! The following structure will store 
! the observation details for plotting.
!**************************************
    type OBSERVATIONDETAILS 
        character(len=40) telescope 
        character(len=40) instrument      
        character(len=40) obsID
        character(len=40) expID
        character(len=40) startObs
        character(len=40) endObs
        real              exposure
    end type OBSERVATIONDETAILS


!*************************************
! The following stores the sizes of
! characters, etc, that will be used
! by pgplot.
!************************************
    type PGPLOTSIZES
        real charSize
        real tickSize
        real labelSize
    end type PGPLOTSIZES

!***********************************
    type POINT
        real(kind=single) ::  x
        real(kind=single) :: y1
        real(kind=single) :: y2
       
    end type POINT


    private 

    public rgsimplot
        type(PGPLOTSIZES)            :: pgplotCharSize
        integer :: maxNoColours = 10
        integer :: colourIndex(1: 10)
        logical :: display = .false.

contains 
  
!***************************************************
! subroutine rgsimplot
!***************************************************
subroutine rgsimplot
  
    real, allocatable            :: image(:, :)         ! spatial image array
    integer                      :: dim(2)              ! image dimension array
    character(len=255)           :: extName 
 
    character(len=1024)          :: xLabel              ! plotting labels
    character(len=1024)          :: yLabel              ! plotting labels

    character(len=1024)          :: spatialSet          ! input spatial filename
    character(len=1024)          :: enDispSet           ! input energy dispersion filename
    character(len=1024)          :: srcListSet          ! input source list set  

    character(len=1024)          :: device, cdevice     ! ploting device e.g /xw
    integer                      :: colour              ! Produce a colour output?
      
    character(len=40)            :: colourMap              ! LIN/SQRT/LOG colour mapping
    logical(kind=Bool)           :: invert                 ! Invert apcolour map?
    logical(kind=Bool)           :: withSpatialSet         ! Plot spatial--dispersion image?
    logical(kind=Bool)           :: withEnDispSet          ! Plot energy--dispersion image?
    logical(kind=Bool)           :: withSpatialRegionSets  ! Plotspatial extraction regions?
    logical(kind=Bool)           :: withEnDispRegionSets   ! Plot order extraction regions?  

    type(DataSetT)               :: inDataSet           ! pointer to event list
    type(ArrayT)                 :: inArray             ! pointer to event list table

    type(DataSetT)               :: srcListSetH         ! Input dataset handle.    
    type(TableT)                 :: srcTableH           ! Input table handle.

    integer                      :: numXBins, numYBins 
    real                         :: XBINWIDTH, YBINWIDTH 

    integer                      :: i, l                ! loop counters
    integer                      :: pgbeg
    real(kind=single)            :: paperXmin
    real(kind=single)            :: paperYmin
    real(kind=single)            :: paperXmax
    real(kind=single)            :: paperYmax
    real(kind=single)            :: xMin               ! 
    real(kind=single)            :: yMin, yMax                !  
    character(len=1024)          :: plotFile            !  output filename
                
    type(OBSERVATIONDETAILS)     :: obsDetails  

    integer                      :: order(10)
    integer                      :: noOrders
    real(kind=single), allocatable, dimension(:) :: sourceRA, sourceDec
    character(len=20), allocatable, dimension(:) :: sourceName

    integer, allocatable, dimension(:) :: sourceNumber


    integer :: noSources = 0                     


    !***************************************************
    ! Get the input parameters and make relevant checks.
    !***************************************************
    withSpatialSet = booleanParameter("withspatialset") 
    withEnDispSet  = booleanParameter("withendispset")    

    if ((.not. withSpatialSet) .and. (.not. withEnDispSet)) then
        call fatal("Nothing to plot");
    end if

    if ( withSpatialSet ) then
        spatialSet = stringParameter("spatialset")    
        if (spatialSet .eq. "" ) then
            call fatal("No spatial set supplied");
        end if
    end if
    
 
    if ( withEnDispSet ) then
        enDispSet = stringParameter("endispset") 
        if (enDispSet .eq. "" ) then
            call fatal("No Energy Dispersion set supplied");
        end if
    end if                                    



    withSpatialRegionSets = booleanParameter("withspatialregionsets")
    withEnDispRegionSets  = booleanParameter("withendispregionsets")        

    if (withSpatialRegionSets .or. withEnDispRegionSets) then
        srcListSet = stringParameter("srclistset")
        if (srcListSet .eq. "" ) then
            call fatal("No source list set provided");
        end if
    end if


               

    plotFile = trim(stringParameter("plotfile"))
    device = trim(stringParameter("device")) ! Gets the device and converts it to lower-case.
    call  ConvertStringToLowerCase(device) ! Converts it to lower-case.


  
    if ( trim(device(1 : 2)) .ne. "/x" ) then
        if(trim(plotfile) .eq. "") then
            call fatal("No plot file supplied", "Hard copy device selected")
        else                        ! Check file doesn't exist if SAS_CLOBBER set       
            call CheckClobber(plotfile)         
        end if
        cdevice = trim(plotFile) // trim(device);
    else
        cdevice = trim(device)
        display = .true.         
    end if
    if (trim(device) .eq. "/gif") display = .true.
    invert = booleanParameter("invert")

    colour = intParameter("colour")
  
    colourMap = stringParameter("colourmap")
    call ConvertStringToLowerCase(colourMap)

    paperXmin = realParameter("paperxmin")
    paperYmin = realParameter("paperymin")
    paperXmax = realParameter("paperxmax")
    paperYmax = realParameter("paperymax")

    if (abs(paperXmin - paperXmax) < 0.001) then 
        call fatal("Invalid graph size", "paperxmin=paperxmax")
    end if  
    if (abs(paperYmin -  paperYmax) < 0.001) then
        call fatal("Invalid graph size", "paperymin=paperymax")
    end if
  
    call message(AppMsg, Verbose, "parameters OK") 
    
    
    !  Scale the size of characters, etc, to the drawing area size.
    call SetPgPlotSizes(paperXmin, paperXmax, paperYmin, paperYmax)  




    !***********************************************
    ! Finished getting and checking input parameters 
    !
    ! Open the pgplot plotting device
    !***********************************************  

    if (pgbeg(0, cdevice, 1, 1) .ne. 1) then
        call fatal("badDevice","Unrecognized PGPLOT device")
    end if   

  
    if (withSpatialSet .and. withEnDispSet) then ! Split the page up into 2
        call pgsubp(1, 2)                        ! drawing areas.
        call pgscf(2)
    end if



    if (withspatialregionsets .or. withendispregionsets) then        
        !************************************************
        ! Get the source coordinates for each source
        !************************************************
        noSources = parametercount("srcidlist") 

        allocate(sourceNumber(1 : noSources))
        allocate(sourceRA(1 : noSources))
        allocate(sourceDec(1 : noSources)) 
        allocate(sourceName(1 : noSources)) 




        srcListSetH = dataSet(srcListSet, read)  
        if (noSources .gt. maxNoColours) then
            call warning( "Too many regions", "Unable to draw more than 11 fits regions in different colours" &
                          // "The remainder will be drawn in black")
        end if 




        call GetSourceDetails(srcListSetH, noSources, sourceNumber, &
                            sourceRA, sourceDec, sourceName)
    end if 
  
    call SetUpColourIndices


    !*************************** 
    ! Set up the color map 
    !**************************       
    call SetUpPalette(colour)

    !************************
    !  Set up colour table.
    !****************************
    select case (trim(colourMap))
        case ("log")
            call pgsitf(1)
        case ("sqrt") 
            call pgsitf(2)
        case default
            call pgsitf(0)  ! Linear
    end select



!**********************************************
! Start off SpatialSet block
!*********************************************
SPATIALSETBLOCK: if (withSpatialSet) then

      ! Get a readonly handle on the spatial FITS image
      inDataSet = dataSet(spatialSet, read)                       
       
      ! Get the observation details for plotting on the graph from the file  
      call GetObservationDetails(inDataSet, obsDetails)

      ! Get a handle on the image array extension.
      inArray = array(inDataSet, "PRIMARY", read)
  
      call GetArrayAttributes(inArray, xLabel, yLabel, xMin, &
                              yMin, xBinWidth, yBinWidth)
    
    

      ! Get the number of bins.
      dim = dimensions(inArray)

      numXBins = dim(1)
      numYBins = dim(2)


      ! Get a pointer to the image.
      allocate(image(1 : numXBins,  1 : numYBins))
      call GetImage(image, inArray)
   
      ! Finished with data file
      call release(inArray)
      call release(inDataSet)






      ! Clear the screen. Set up window and viewport.
      call pgpage
      call pgsvp(paperXmin, paperXmax, paperYmin, paperYmax)

     
      !*************************************      
      ! Plot the image
      !*************************************
   
      yMax = yMin + real(numYBins) * real(yBinWidth)        


      call PlotImage(image, NumXBins, NumYBins, xMin, XBinWidth, &
                 yMin, yMax, YBinWidth, colourMap, invert, colour)
                        
      !*******************
      ! Set up labels.    
      !******************* 
      call SetXLabel(xLabel)

      select case(trim(ylabel))   
          case("XDSP_CORR")      
              yLabel = "CROSS DISPERSION ANGLE (radians)"

          case("PHA") 
              yLabel = "PHA (Channel)"

          case("PI") 
              yLabel = "PI (Channel)"

          case( "PIFUDGE") 
              yLabel = "PI (Channel)"

      end select
      !*********************************************
      ! Plot the labels, etc, for the spatial image.
      !**********************************************
      call PlotExposureDetails(xLabel, yLabel, "", &
          "Spatial Image", obsDetails)
            







 
!****************************************************
! Start of spatial region sets block
!****************************************************
SPATIALREGIONSETSBLOCK: if ( withspatialregionsets) then

        if (hasBlock(srcListSetH, "SRCLIST")) then  ! Was SOURCES           

!            call  PlotBackGroundRegion(srcListSet, srcListSetH, obsDetails%instrument)
            if (noSources .gt. maxNoColours) then
                call warning("Too many regions", "Unable to draw more than 11 fits regions in different colours"&
                             //"The remainder will be drawn in black")
            end if 
    


              
            do i = 1, noSources

 

                call CreateExtName1(sourceNumber(i), obsDetails, extName)
         
                if (hasBlock(srcListSetH, trim(extname))) then                 
                                  
                    call PlotFitsRegion(srcListSet, srcListSetH, extName, i, 1, "XDSP_CORR")
                      

                else
                
                    write(errstr,*) "Source region extension ",&
                          trim(extname)," does not exist in sourcelist, no region plotted"
                    call warning("Block not present", trim(errstr))
                    
                end if

            end do
        end if  
        
        call PlotSourceDetails(noSources, sourceRA, sourceDec, sourceName, .false.)
   
             
  
        end if SPATIALREGIONSETSBLOCK

        deallocate(image)
    


      !************************************************
      ! End of spatial set block
      !***********************************************     


    end if SPATIALSETBLOCK 

    !**********************************************
    ! Orders image
    !**********************************************
 
  if (withEnDispSet) then
      
      ! Get a readonly handle on the spatial FITS image
      inDataSet = dataSet(enDispSet, read)                       

      ! Get observation identification details for plotting
      call GetObservationDetails(inDataSet, obsDetails)

    
      ! Get a handle on the image array extension.
      inArray = array(inDataSet, "PRIMARY", read)
  
      call GetArrayAttributes(inArray, xLabel, yLabel, xMin, &
                              yMin, xBinWidth, yBinWidth)
   
      ! Get the number of bins.
      dim = dimensions(inArray)

      numXBins = dim(1)
      numYBins = dim(2)

      ! Get a pointer to the image.
      allocate(image(numXBins, numYBins))

      call GetImage(image, inArray)

      ! Finished with data file
      call release(inarray)
      call release(inDataSet)

    
      ! Clear the screen. Set up window and viewport.
   
      call pgpage
      call pgsvp(paperXmin, paperXmax, paperYmin, paperYmax)
        
      !***********************
      ! Plot the image
      !***********************
      yMax = yMin + real(numYBins) * real(yBinWidth) 

      call PlotImage(image, NumXBins, NumYBins, xMin, XBinWidth, &
                 yMin, yMax, YBinWidth, colourMap, invert, colour)
   
    
      ! Set up labels.
      call SetXLabel(xLabel)


      select case(trim(yLabel))
          case("DETY")     
              yLabel = "DETY (mm)"

          case("PHA") 
              yLabel = "PHA (Channel)"

          case("PI") 
              yLabel = "PI (Channel)"

          case( "PIFUDGE")
          yLabel = "PI (Channel)"

      end select

      !**************************************************
      ! Plot the graph labels, etc, for the orders image 
      !**************************************************        
      call PlotExposureDetails(xLabel, yLabel, "",&
              "Orders Image", obsDetails)
      
      noOrders = parametercount("orderlist") 
                            
      call GetOrders(noOrders, order)
           
      if (withendispregionsets) then 
           
          if (hasBlock(srcListSetH, "SRCLIST")) then     ! WAS SOURCES and below           
              srcTableH = table(srcListSetH, "SRCLIST")
           
                         
              call release(srcTableH)
              
              
            
              do i = 1, noSources
                
                 !***************************************************
                 ! Loop over orders - either all the ones in the file
                 ! or those specified by the parameter "orderlist"
                 !**************************************************
                  do l = 1, noOrders
                   
                      call CreateExtName2(sourceNumber(i), order(l), obsDetails,  extName)
                      if (hasBlock(srcListSetH, extname)) then 
                
                          
                           call PlotFitsRegion(srcListSet, srcListSetH, extName, i, 1, "PI")                          

                      else
                     
                          write(errstr,*) "Source region extension ", TRIM(extname),&
                              " does not exist in sourcelist, no region plotted"
                          call warning("Block not present", trim(errstr))
                                        
                      end if
                  end do
               
              end do
            
   
          end if 

          !***************************************************************
          ! Plot the source details if the spatial map hasn't been plotted
          !***************************************************************
          if (.not. withSpatialSet) then
              call PlotSourceDetails(noSources, sourceRA, sourceDec, sourceName, .false.)
          end if 
   

        end if    
     
        deallocate(image)
    end if

    if (withSpatialRegionSets .or. withEnDispRegionSets) call release(srcListSetH)
 
    if (allocated(sourceNumber)) deallocate(sourceNumber)
    if (allocated(sourceRA))     deallocate(sourceRA)
    if (allocated(sourceDec))    deallocate(sourceDec)
    if (allocated(sourceName))   deallocate(sourceName)

    call PlotProgramVersionNo 

    call pgend

end subroutine rgsimplot

!******************************************
! ConvertStringToLowerCase
! Converts string chararacters to lower-case
!*****************************************
subroutine  ConvertStringToLowerCase(string)
   
   character(len=*), intent(inout) :: string
   integer                     :: i
   integer                     :: newch
   character(len=1)            :: cha
   
   do i = 1, len(string)
      cha = string(i : i)
      if ( lge(cha, 'A') .and. lle(cha, 'Z') ) then
         newch = ichar(cha) + ichar('a') - ichar('A')
         string(i : i) = char(newch)
      end if
   end do
   
end subroutine ConvertStringToLowerCase


!******************************************************************
! subroutine SetUpPalette
! Set a "palette" of colors in the range of color indices used by
! PGIMAG.
!******************************************************************   
subroutine SetUpPalette(type)
    integer, intent(in) :: type

    real        :: GL(2),  GR(2),  GG(2),  GB(2)
    real        :: RL(9),  RR(9),  RG(9),  RB(9)
    real        :: HL(5),  HR(5),  HG(5),  HB(5)
    real        :: WL(10), WR(10), WG(10), WB(10)
    real        :: AL(20), AR(20), AG(20), AB(20)
    real        :: TL(4),  TR(4),  TG(4),  TB(4)

    DATA GL /0.0, 1.0/
    DATA GR /0.0, 1.0/
    DATA GG /0.0, 1.0/
    DATA GB /0.0, 1.0/

    DATA RL /-0.5, 0.0, 0.17, 0.33, 0.50, 0.67, 0.83, 1.0, 1.7/
    DATA RR / 0.0, 0.0,  0.0,  0.0,  0.6,  1.0,  1.0, 1.0, 1.0/
    DATA RG / 0.0, 0.0,  0.0,  1.0,  1.0,  1.0,  0.6, 0.0, 1.0/
    DATA RB / 0.0, 0.3,  0.8,  1.0,  0.3,  0.0,  0.0, 0.0, 1.0/

    DATA HL /0.0, 0.2, 0.4, 0.6, 1.0/
    DATA HR /0.0, 0.5, 1.0, 1.0, 1.0/
    DATA HG /0.0, 0.0, 0.5, 1.0, 1.0/
    DATA HB /0.0, 0.0, 0.0, 0.3, 1.0/

    DATA WL /0.0, 0.5, 0.5, 0.7, 0.7, 0.85, 0.85, 0.95, 0.95, 1.0/
    DATA WR /0.0, 1.0, 0.0, 0.0, 0.3,  0.8,  0.3,  1.0,  1.0, 1.0/
    DATA WG /0.0, 0.5, 0.4, 1.0, 0.0,  0.0,  0.2,  0.7,  1.0, 1.0/
    DATA WB /0.0, 0.0, 0.0, 0.0, 0.4,  1.0,  0.0,  0.0, 0.95, 1.0/

    DATA AL /0.0, 0.1, 0.1, 0.2, 0.2, 0.3, 0.3, 0.4, 0.4, 0.5, &
            0.5, 0.6, 0.6, 0.7, 0.7, 0.8, 0.8, 0.9, 0.9, 1.0/
    DATA AR /0.0, 0.0, 0.3, 0.3, 0.5, 0.5, 0.0, 0.0, 0.0, 0.0, &
            0.0, 0.0, 0.0, 0.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0/
    DATA AG /0.0, 0.0, 0.3, 0.3, 0.0, 0.0, 0.0, 0.0, 0.8, 0.8, &
            0.6, 0.6, 1.0, 1.0, 1.0, 1.0, 0.8, 0.8, 0.0, 0.0/
    DATA AB /0.0, 0.0, 0.3, 0.3, 0.7, 0.7, 0.7, 0.7, 0.9, 0.9, &
            0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0/

    DATA TL /0.0, 0.5, 0.5, 1.0/
    DATA TR /0.2, 0.6, 0.6, 1.0/
    DATA TG /0.0, 0.0, 0.5, 1.0/
    DATA TB /1.0, 0.0, 0.0, 0.0/
   
   
    select case (type)
        case (1) !        -- gray scale

            call pgctab(GL, GR, GG, GB, 2, 1.0, 0.5)  ! Were all 1.0, 0.5
           
        case (2) !        -- rainbow

            call pgctab(RL, RR, RG, RB, 9, 1.0, 0.5)
            
        case (3) !        -- heat
            call pgctab(HL, HR, HG, HB, 5, 1.0, 0.5)
  
        case (4) !        -- weird IRAF
            call pgctab(WL, WR, WG, WB, 10, 1.0, 0.5)
  
        case (5) !        -- AIPS
            call pgctab(AL, AR, AG, AB, 20, 1.0, 0.5)

        case (6) !        -- TJP
            call pgctab(TL, TR, TG, TB, 4, 1.0, 0.5)
 
      end select

end subroutine SetUpPalette
!*************************************************************
!
! Subroutine DrawColourScale
!
! Plot an annotated grey-scale or color wedge parallel to a given axis
! of the the current viewport. This routine is designed to provide a
! brightness/color scale for an image drawn with PGIMAG or PGGRAY.
! The wedge will be drawn with the transfer function set by PGSITF
! and using the color index range set by PGSCIR.
!
!
! Arguments:
!  SIDE   (input)  : The first character must be one of the characters
!                    'B', 'L', 'T', or 'R' signifying the Bottom, Left,
!                    Top, or Right edge of the viewport.
!                    The second character should be 'I' to use PGIMAG
!                    to draw the wedge, or 'G' to use PGGRAY.
!  DISP   (input)  : the displacement of the wedge from the specified
!                    edge of the viewport, measured outwards from the
!                    viewport in units of the character height. Use a
!                    negative value to write inside the viewport, a
!                    positive value to write outside.
!  WIDTH  (input)  : The total width of the wedge including annotation,
!                    in units of the character height.
!  foreGroundValue   (input)  : The value which is to appear with shade
!                    1 ("foreground"). Use the values of minValue and maxValue
!                    that were supplied to PGGRAY or PGIMAG.
!  backGroundValue   (input)  : the value which is to appear with shade
!                    0 ("background").
!  LABEL  (input)  : Optional units label. If no label is required
!                    use ' '.
!
!*************************************************************
subroutine DrawColourScale(side, disp, width, label, &
                           foreGround, backGround)
    character(len = *), intent(in) :: side   ! The side of the graph where the colour scale goes
    character(len = *), intent(in) :: label  ! The label to be plotted
    
 
    real(kind = single), intent(in) :: disp   ! The displacement of the scale from the asix
    real(kind = single), intent(in) :: width  ! The width of the colour scale
    real(kind = single), intent(in) :: foreGround
    real(kind = single), intent(in) :: backGround
 
   
    real xcharSize, ycharSize  ! Stores the width and height of a character


    call SetUpColourIndices
  
    call pgsci(1) ! was 10      
    call pgsch(pgplotCharSize%tickSize)
     
!    call pgwedg("TI", 0.0, width, foreGround, backGround, "")   ! was 0.1-1.0
   
    call pgwedg(trim(side), disp, width, foreGround, backGround, "")   ! was 0.1-1.0

    
    


    call pgsch(pgplotCharSize%labelSize * 0.75)  ! Set the label size
    call pgqcs(0, xcharSize, ycharSize)
    call pgslw(2)
    
    call pgmtxt(side, disp + width + ycharSize * 100.0, 0.5, 0.5, trim(label))
    call pgslw(1)
    
end subroutine DrawColourScale


!*******************************************************
! plotFitsRegion
!
!******************************************************
subroutine PlotFitsRegion(fileName,    &
                          datasetH,    &
                          extName,     &
                          ci,          &
                          lineStyle,   & 
                          yColumnName)
    character(len=*), intent(in) :: fileName
    character(len=*), intent(in) :: extName
    integer, intent(in) :: ci, lineStyle
    type(DataSetT), intent(in) :: dataSetH
    character(len=*), intent(in) ::  yColumnName
    type(TableT) :: tableSetH
    character(len=20) :: xColumnName
    
    integer nPoints, nregions
    
   
    integer i, j
    integer :: nRows
    real(kind = single), pointer :: yval(:, :), xval(:, :)
    type(POINT), dimension(:), allocatable :: data    
    type(ColumnT) :: shapeCol, xCol, yCol
    
    real(kind=single) :: x1, x2, y2, dx, dy, yCentre1, yCentre2, yDiff
    integer(kind=int32) :: nData 
    logical :: status

    ndata=0

    call SetUpColourIndices


    call pgsls(lineStyle)
    if (ci .gt. maxNoColours) then
        call pgsci(colourIndex(maxnocolours)) 
    else
        call pgsci(colourIndex(ci)) 
    end if
    tableSetH = table(dataSetH, TRIM(extName))
    if (hasColumn(tableSetH,"LAMBDA")) then
       xColumnName = "LAMBDA"
    else
       xColumnName = "BETA_CORR"
    endif
    


    nRows = numberOfRows(tableSetH)
    if (nRows .eq. 0) then
      
        write(errstr, *) "No rows of data in file ", trim(fileName)
        call warning("Cannot draw fits region", trim(errstr))
        call release(tableSetH)
      
        return
    end if
   
    shapeCol = column(tableSetH, "SHAPE", read)

    xCol = column(tableSetH, xColumnName, read)   ! was X
    xval => real32VectorData(xCol)

    yCol = column(tableSetH, yColumnName, read)   ! was y
    yval => real32VectorData(yCol)
   

    nPoints = ubound(xval, 1) - lbound(xval, 1) + 1


    nregions = ubound(xval, 2) - lbound(xval, 2) + 1


    ndata = ndata + 1
    do i = 0, nregions - 1   ! was 1
    
        do j = 1, npoints - 1
            if (xval(j, i) .lt. 1e-6) exit 

            
 
            if (abs(xval(j, i) - xval(j - 1, i)) .lt. 1e-6) then

            
                nData = nData + 1
            end if  
        end do

    end do

    allocate(data(1 : ndata))
    
    nData = 0



    !********************************
    ! Draw the regions
    !********************************


    do i = 0, nregions - 1   ! was 1

        !*************************************
        ! Plot the points for this region
        !*************************************

        call pgmove(xval(0, i), yval(0, i))
        

        do j = 1, npoints - 1
            if (xval(j, i) .lt. 1e-6) exit 

            
 
            if (abs(xval(j, i) - xval(j - 1, i)) .lt. 1e-6) then

                !**********************************
                ! Drawing a vertical line
                !**********************************           
 
                call pgmove(xval(j, i), yval(j, i))

                nData = nData + 1
                data(nData) =  POINT(xval(j, i), yval(j, i), yval(j - 1, i))
 
            else 
               
                call pgdraw(xval(j, i), yval(j, i))
                                   
            end if 
        end do

    end do

    call SortPoints(nData, data)

    call pgqwin(x1, x2, y2, y2)
    dx = abs(x2 - x1) / 50.0
    
    call pgmove(data(1)%x, data(1)%y1)
    call pgdraw(data(1)%x, data(1)%y2)
   


    do i = 2, nData - 1
              
        yCentre1 = (data(i)%y1 + data(i)%y2) / 2.0
        dy = abs(data(i)%y1 - data(i)%y2) / 5.0
        status = .false.
        do j = 1, nData - 1
            if (i .eq. j) cycle
            yCentre2 = (data(j)%y1 + data(j)%y2) / 2.0
    
            yDiff = abs(yCentre1 - yCentre2)
            if (abs(data(i)%x - data(j)%x) .lt. dx .and. yDiff .lt. dy) then
                status = .true.
                exit
            end if 
      
        end do  
        if (status) cycle
        call pgmove(data(i)%x, data(i)%y1)
        call pgdraw(data(i)%x, data(i)%y2)
 
        
    end do        
    call pgmove(data(nData)%x, data(nData)%y1)
    call pgdraw(data(nData)%x, data(nData)%y2)
 

    call release(tableSetH)
  
    call pgsci(1) ! was 10
    call pgsls(1)

   

    call release(shapeCol)       
    call release(xCol)
    call release(yCol)
    deallocate(data)
    
end subroutine plotFitsRegion
!*******************************************************
! PlotExposureDetails - Adds the exposure details, 
!            etc, to the graph
!********************************************************
subroutine PlotExposureDetails(xLabel, yLabel, title, &
              imageType, obsDetails)

   implicit none
   
   character(*), intent(in) :: xLabel            ! Graph x-label
   character(*), intent(in) :: yLabel            ! Graph y-label
   character(*), intent(in) :: title             ! Graph title
   character(*), intent(in) :: imageType
 
   type(OBSERVATIONDETAILS), intent(in) ::  obsDetails 
   
   character(len=100) :: text, expText

   call SetUpColourIndices
    
!   call pgsci(colourIndex(10))
   call pgsci(1) 
   call pgslw(2)
   call pgsch(pgplotCharSize%labelSize)

   
   call pglab("", trim(yLabel), trim(title))  ! Draw the graph labels.  

   call pgmtxt("b", pgplotCharSize%labelSize * 2.0, 0.5, 0.5, trim(xlabel))
 

   call pgslw(2)
   text = trim(obsDetails%telescope)//"-Newton  "&
        //trim(obsDetails%instrument)//" "//&
        imageType
   call pgmtxt('T', 3.0, 0.5, 0.5, text)
   call pgsls(2)
    
   call pgsch(pgplotCharSize%labelSize * 0.75)
    
   text = "DATE-OBS "// trim(obsDetails%startObs) // &
         "   DATE-END "// trim(obsDetails%endObs)
   call pgmtxt('T', 2.0, 0.5, 0.5, text)
  
   text = "OBS-ID " // obsDetails%expID(1 : 10) // " EXP-NO " // &
           obsDetails%expID(11 : 13)


   if (obsDetails%exposure > 0.0) then  ! Add the exposure length if it is known
       write(expText, '(e14.8)') obsDetails%exposure
       text = trim(text)// "   EXPOSURE "// trim(expText)
   end if

   call pgmtxt('T', 1.0, 0.5, 0.5, text)    
   call pgslw(1)
   call pgsls(1)

end subroutine PlotExposureDetails


!**************************************************
! Subroutine TransformMatrix
! Sets up plotting transformation matrix.
! Added by CB to prevent the code having to be 
! repeated twice in the main program 
! (as in original).
!**************************************************
subroutine TransformMatrix(trimage, xmin, &
                        xbinWidth, ymin, ybinWidth)
    real(kind=single), dimension(6), intent(inout) :: trimage
    real(kind=single), intent(in) :: xmin
    real(kind=single), intent(in) :: xbinWidth
    real(kind=single), intent(in) :: ymin
    real(kind=single), intent(in) :: ybinWidth

    trimage(1) = xmin - xbinWidth / 2.0
    trimage(2) = xbinWidth
    trimage(3) = 0.0
    trimage(4) = ymin - ybinWidth / 2.0
    trimage(5) = 0.0
    trimage(6) = ybinWidth

end subroutine TransformMatrix

!************************************************
! CreateExtName1
!***********************************************
subroutine CreateExtName1(sourceID, obsDetails, extName)
    integer, intent(in) :: sourceID
    type(OBSERVATIONDETAILS), intent(in) :: obsDetails
    character (len=*), intent(inout) :: extName
    character(len = 20) :: format(5) = &
           (/'(a4, a4, i1, a8)', &
             '(a4, a4, i2, a8)', &
             '(a4, a4, i3, a8)', &
             '(a4, a4, i4, a8)', &
             '(a4, a4, i5, a8)'/)



    write(extName, format(int(log10(real(sourceID))) + 1)) &
              trim(obsDetails%instrument),         &
              "_SRC", sourceID, "_SPATIAL"
    


end subroutine CreateExtName1

!************************************************
! CreateExtName2
!***********************************************
subroutine CreateExtName2(sourceNumber, order, obsDetails, extName)
    integer, intent(in) :: sourceNumber, order
    type(OBSERVATIONDETAILS), intent(in) :: obsDetails
    character (len=*), intent(inout) :: extName
    character(len = 20) :: format(5) =  &
              (/'(a4, a4, i1, a7, i1)', &
                '(a4, a4, i2, a7, i1)', &
                '(a4, a4, i3, a7, i1)', &
                '(a4, a4, i4, a7, i1)', &
                '(a4, a4, i5, a7, i1)'/)




    write(extName, format(int(log10(real(sourceNumber))) + 1)) &
          trim(obsDetails%instrument),              &
          "_SRC", sourceNumber, "_ORDER_", order
   


end subroutine CreateExtName2

!*************************************************
! Scales the font sizes,etc,  to the size of the 
! drawing area.
!
!************************************************
subroutine  SetPgPlotSizes(xl, xu, yl, yu)
    real(kind=single), intent(in) :: xl, xu, yl, yu
  
      
    pgplotCharSize%charSize = (xu - xl) * (yu - yl) * 2.0
  
    if (pgplotCharSize%charSize < 0.4)  pgplotCharSize%charSize = 0.4
    if (pgplotCharSize%charSize > 1.1)  pgplotCharSize%charSize = 1.1

    pgplotCharSize%tickSize = pgplotCharSize%charSize * 1.2  ! Tick and axes values sizes.
    pgplotCharSize%labelSize = min(1.2, 1.1 * pgplotCharSize%charSize) ! Character size of labels.

 

end subroutine SetPgPlotSizes
 
!**********************************************
! CheckClobber
!    If SAS_CLOBBER = 0 then checks to see if
!    fileName already exists.
!*********************************************
subroutine CheckClobber(fileName)
    character(len=*), intent(in) :: fileName
    
    if (.not. clobber()) then
        open(unit = 2, FILE=filename, status = 'OLD', err = 1)
        close(unit = 2)
        call fatal("Plot file already exists", "SAS-CLOBBER set")
    1   return          !   File doesn't exist - okay to continue
    end if

    
end subroutine CheckClobber
!*************************************************************
! Subroutine GetObservationDetails
! Retrieves the observation details from a fits file
! for plottiong on the graph
!************************************************************
subroutine GetObservationDetails(dataSet, obsDetails)
    type(DataSetT), intent(in) :: dataSet
    type(OBSERVATIONDETAILS), intent(inout) :: obsDetails  

    call GetStringAttribute(dataSet, "TELESCOP", obsDetails%telescope)
    call GetStringAttribute(dataSet, "INSTRUME", obsDetails%instrument)
    call GetStringAttribute(dataSet, "OBS_ID",   obsDetails%obsID)
    call GetStringAttribute(dataSet, "EXP_ID",   obsDetails%expID)
    call GetStringAttribute(dataSet, "DATE-OBS", obsDetails%startObs)
    call GetStringAttribute(dataSet, "DATE-END", obsDetails%endObs)
    call GetRealAttribute  (dataSet, "EXPOSURE", obsDetails%exposure)
end subroutine GetObservationDetails

!***********************************************************
! Subroutine GetStringAttribute - Attempts to read a string attribute
!                        from a fits file
!                        (Added by CB on 14.09.2000) because
!                        apparently some attributes may
!                        not be present.
!***********************************************************
subroutine GetStringAttribute(dataSet, attribute, string)
    type(DataSetT), intent(in) :: dataSet
    character(len=*), intent(in) :: attribute
    character(len=*), intent(inout) :: string

    if (.not. hasAttribute(dataSet, attribute)) then
!        call warning("Attribute not found", attribute);
        string = ""
    else
        string = stringAttribute(dataSet, attribute)
    end if
end subroutine GetStringAttribute

!***********************************************************
! Subroutine GetRealAttribute - Attempts to read a real attribute
!                        from a fits file
!                        (Added by CB on 14.09.2000) because
!                        apparently some attributes may
!                        not be present.
!***********************************************************
subroutine GetRealAttribute(dataSet, attribute, number)
    type(DataSetT), intent(in) :: dataSet
    character(len=*), intent(in) :: attribute
    real, intent(inout) :: number

    if (.not. hasAttribute(dataSet, attribute)) then
        number= 0.0
    else
        number = real32Attribute(dataSet, attribute)
    end if
   
end subroutine GetRealAttribute
!****************************************************************
! subroutine PlotImage
! Plots the 2-d image and draws the colour scale wedge
!
!****************************************************************
subroutine PlotImage(image, numXBins, numYBins, xMin, xBinWidth, &
                     yMin, yMax, yBinWidth, colourMap, invert, colour)
    real(kind=single), intent(inout), dimension(:, :) :: image
    integer, intent(in) :: numXBins
    integer, intent(in) :: numYBins
    real(kind=single), intent(in) :: xMin
    real(kind=single), intent(in) :: xBinWidth
    real(kind=single), intent(in) :: yMin, yMax
    real(kind=single), intent(in) :: yBinWidth
    character(len=*), intent(in)  :: colourMap
    logical(kind=Bool), intent(in) :: invert
    integer, intent(in) :: colour
    character(len = 80) :: label
    real(kind=single), dimension(1 : 6) :: imageTransMatrix
    real(kind=single) :: minValue
    real(kind=single) :: maxValue
    real(kind=single) :: xMax
   
    real(kind=single) :: temp

    xMax = xMin + real(numXBins) * real(xBinWidth) 
 
    call pgswin(xMin, xMax, yMin, yMax)
  
    call EffectiveRange(image, 0.05, 0.0005, minValue, maxValue)  ! Second parameter was 0.05


    !**************************
    ! Invert values if required
    !************************** 
    if(invert) then
        temp = minValue
        minValue = maxValue
        maxValue = temp
    end if

    !*************************** 
    ! Set up the color map 
    !**************************       
    call SetUpPalette(colour)
    !************************
    !  Set up colour table.
    !****************************
    select case (trim(colourMap))
        case ("log")
            call pgsitf(1)
        case ("sqrt") 
            call pgsitf(2)
        case default
            call pgsitf(0)  ! Linear
    end select

   
    !*************************************
    ! Set up plotting transform matrix.
    !*************************************
    call TransformMatrix(imageTransMatrix, xMin, xBinWidth, yMin, YBinWidth)
   
    !*********************************
    ! Draw the map using pgplot pgimag.  
    !*********************************        
    call pgimag(image, numXBins, numYBins, 1, NumXBins, 1, &
                  numYBins, minValue, maxValue, imageTransMatrix)
     
    !***************************
    ! Draw the colour-scale bar.
    !***************************

    label = "Counts per binned pixel (" // trim(colourMap) // " intensity)" 
    call DrawColourScale("RI", 0.75, 2.5, label, minValue, maxValue)
   

    call pgscf(2)

    call pgslw(2)  
    call pgsls(1)  
    call pgsch(pgplotCharSize%tickSize)           ! Tick mark size
    call pgbox("bctsi", 0.0, 0, "bctsi", 0.0, 0)  ! NB- "i" inverts the tick marks
   
   
    call pgsch(pgplotCharSize%charSize)           ! Character size
    call pgbox("n", 0.0, 0,"n", 0.0, 0)  
    call pgslw(1)

end subroutine PlotImage
!*******************************************
! subroutine GetImage
! Gets the 2-d image from the fits file
!*******************************************
subroutine GetImage(image, inArray)
    real(kind=single), intent(inout), dimension(: , :) :: image
    type(ArrayT) :: inArray    
    integer(kind=int8), dimension(:, :),  pointer :: int8ImageP     ! Integer8 array pointer.
    integer(kind=int32), dimension(:, :), pointer :: int32ImageP    ! Integer32 array pointer.
    integer(kind=int16), dimension(:, :), pointer :: int16ImageP    ! Integer16 array pointer.
    real(kind=single), dimension(:, :),   pointer :: real32ImageP   ! Real32 array pointer. 
    real(kind=double), dimension(:, :),   pointer :: real64ImageP   ! Real64 array pointer.
    
    select case(arrayDataType(inArray))
        case(integer8)   
            int8ImageP => int8Array2Data(inArray)
            image = real(int8ImageP)
    
        case(integer16)
            int16ImageP => int16Array2Data(inArray)
            image = real(int16ImageP)    

        case(integer32) 
            int32ImageP => int32Array2Data(inArray)
            image = real(int32ImageP)

        case(real32)
            real32ImageP => real32Array2Data(inArray)
            image = real32ImageP    

        case(real64)
            real64ImageP => real64Array2Data(inArray)
            image = real64ImageP

    end select
   

end subroutine GetImage
!********************************************
! subroutine GetArrayAttributes
!********************************************
subroutine GetArrayAttributes(inArray, xLabel, yLabel, xMin, &
                              yMin, xBinWidth, yBinWidth)
    type(ArrayT)                     :: inArray
    character(len = *), intent(out)  :: xLabel
    character(len = *), intent(out)  :: yLabel
    real(kind = single), intent(out) :: xMin
    real(kind = single), intent(out) :: yMin
    real(kind = single), intent(out) :: xBinWidth
    real(kind = single), intent(out) :: yBinWidth
    real(kind=single) :: xref
    real(kind=single) :: yref

    xLabel = stringAttribute(inArray,    "CTYPE1")
   
    if (trim(xLabel) == "M_LAMBDA") then
       xLabel = "LAMBDA"
    endif

    yLabel = stringAttribute(inArray,    "CTYPE2")
    xref = real32Attribute(inArray,      "CRPIX1")
    yref = real32Attribute(inArray,      "CRPIX2")
 

    xBinWidth = real32Attribute(inArray, "CDELT1")
    yBinWidth = real32Attribute(inArray, "CDELT2")

    xMin = real32Attribute(inArray,      "CRVAL1") + (1.0 - xref) * xBinWidth
    yMin = real32Attribute(inArray,      "CRVAL2") + (1.0 - yref) * yBinWidth
   
end subroutine GetArrayAttributes
!*************************************************
! SetXLabel
! Sets up the graph x-label 
!************************************************
subroutine SetXLabel(xLabel)
    character(len = *), intent(inout) :: xLabel

    select case(trim(xLabel))
        case( "M_TIMES_LAMBDA") 
            xLabel = "m\gl (nm)"
 
        case( "DETX")
            xLabel = "DETX (mm)"

        case ("BETA_CORR") 
            xLabel = "DISPERSION ANGLE (rad)"   ! was "CORRECTED BETA (radians)"
 
        case ("LAMBDA") 
            xLabel = "LAMBDA (\A)"  
    
    end select

end subroutine SetXLabel

!***************************************
! subroutine ConvertRA
!***************************************
subroutine ConvertRA(ra, hours, minutes, seconds)
    real(kind=single), intent(in) :: ra
    integer, intent(out) ::  hours, minutes
    real, intent(out) :: seconds
    double precision ra1, d

    ra1 = ra / 15.0d0  ! RA in hours
 
    if (ra1 .lt.  0.0d0) ra1 = ra1 + 24.0d0
    if (ra1 .gt. 24.0d0) ra1 = ra1 - 24.0d0
    hours = int(ra1)

    d = (ra1 - dble(hours)) * 60.0d0
    minutes = int(d + 0.00000001d0)
    seconds = ( d - dble(minutes) ) * 60.0d0
    seconds = max(0.0, seconds)
!    write(*, *) "ra=", ra, " Converted Ra = ", hours, minutes, seconds
  
end subroutine ConvertRA

!***************************************
! subroutine ConvertDec
!***************************************
subroutine ConvertDec(dec, degrees, minutes, seconds)
    real(kind=single), intent(in) :: dec
    integer, intent(out) :: degrees, minutes
    real, intent(out) :: seconds
    double precision dec1
    double precision ::  d  
    degrees = int(dec)    
    dec1 = abs(dec)
    d = (dec1 - dble(abs(degrees))) * 60.0d0
    minutes = int(d + 0.00000001d0)
    seconds = (d - dble(minutes)) * 60.0d0
    seconds = max(0.0, seconds)

    if (dec .lt. 0.0d0 .and. degrees .eq. 0) minutes = -minutes
    
    if (dec .lt. 0.0d0 .and. degrees .eq. 0 .and. minutes .eq. 0) seconds = -seconds
 
!    write(*, *) "converted dec = ", degrees, minutes, seconds 

end subroutine ConvertDec

!*****************************************************
! PlotSourceDetails
! For eaxh source in the list, plots its number, 
! RA and dec.
!*****************************************************
subroutine PlotSourceDetails(noSources, &
        sourceRA, sourceDec, sourceName, backGround)
    integer, intent(in) :: noSources
   
    real(kind=single), intent(in), dimension(1 : noSources) :: sourceRA, sourceDec
    character(len=*), intent(in) :: sourceName(1 : noSources)
    logical, intent(in) :: backGround   
    integer :: i, j, length, ypos
    
    integer r1, r2, d1, d2
    real r3, d3
    real x1, x2, y1, y2
    character(len=40) :: string
    real :: rightMargin = 1.0
    real :: step
    real xch, ych    
    character(len=10) :: s1, s2, s3

    if (noSources .eq. 0) return

    call SetUpColourIndices


    call pgqvp(0, x1, x2, y1, y2)  

!    call pgsvp(x1, 0.98, y1, y2) 
    call pgsvp(x1, 0.96, y1, y2) 
   
    call pgslw(2)
  
    call pgsch(pgplotCharSize%charSize * 0.9)

    call pgqcs(4, xch, ych)    
    xch = xch * 0.7
    ych = ych * 0.9

    string = "Name     RA      Declination"
   
  
!    call pgsci(10)
    call pgsci(1)
    if (display) then
        step = 0.05
    else
        step = 1.5
    end if

    ypos = 3
    call pgmtxt("b", real(ypos)+step, rightMargin, 1.0, trim(string))  ! was 0.92

         
    call pgslw(2)
    
    ypos = ypos + 1
    do i = 1, min(3, noSources)
        call pgsci(colourIndex(i))

        call ConvertRA(sourceRA(i), r1, r2, r3)
           
        call ConvertDec(sourceDec(i), d1, d2, d3)
!write(*, *) "ra=", sourceRa(i), " dec=", sourcedec(i)
!write(*, *) "r1=", r1, " r2=", r2, " r3=", r3, " d1=", d1, " d2=", d2, " d3=", d3
        !******************************************
        ! Create the number string
        !****************************************** 
        length = len(trim(sourceName(i)))
        string = ""
        do j = 1, length
            string(12 - length  + j : 12 - length + j) = sourceName(i)(j : j)
        end do 
!        write(string, '(a12)') sourceName(i) 


        !******************************************
        ! Create the RA string
        !******************************************
        if (r1 < 10.0) then    ! Hours
            write(s1, '(a1, i1)')  "0", r1
        else
            write(s1, '(i2)') r1
        end if 

        if (r2 < 10) then    ! Minutes
            write(s2, '(a1, i1)')  "0", r2
        else
            write(s2, '(i2)') r2
        end if 

        if (r3 < 9.95) then    ! Seconds
            write(s3, '(a1, f3.1)')  "0", r3
        else
            write(s3, '(f4.1)') r3
        end if 
                       
        string = trim(string) // " " // trim(s1) // ":" // trim(s2) // ":" // trim(s3) 


        !******************************************
        ! Create the dec string
        !******************************************
        if (d1 < 0 .or. d2 < 0 .or. d3 < 0.0) then
            if (d1 > -10) then   ! Degrees
                write(s1, '(a2, i1)') "-0", abs(d1)
            else 
                write(s1, '(a1, i2)') "-", abs(d1)
            end if
        else
            if (d1 < 10) then
                write(s1, '(a2, i1)') "+0", d1
            else 
                write(s1, '(a1, i2)') "+", d1
            end if
        end if

        d2 = abs(d2)   ! Minutes
        if (d2 < 10) then
            write(s2, '(a1, i1)') "0", d2
        else 
            write(s2, '(i2)') d2
        end if
          
        d3 = abs(d3)   ! Seconds          
        if (d3 < 9.95) then
            write(s3, '(a1, f3.1)') "0", d3  !
        else 
            write(s3, '(f4.1)') d3
        end if
           
        !*********************************************
        ! Plot the source number and its coordinates
        !********************************************* 
        string = trim(string) // " "// trim(s1) // ":" // trim(s2) // ":" // trim(s3)            


!write(*, *) "text=", trim(string)
        call pgmtxt("b", real(ypos)+step, rightMargin, 1.0, trim(string))

        ypos = ypos + 1
       
    end do
    if (backGround) then
        call pgsci(10)

        call pgmtxt("b", real(ypos)+step, rightMargin, 1.0, "Background - - - - -")
    end if

    call pgsfs(2)
    
    call pgslw(1)
    call pgsvp(x1, x2, y1, y2)

end subroutine PlotSourceDetails
!*********************************************
!GetOrders
!*********************************************
subroutine  GetOrders(noOrders, order) 
    integer, intent(in) :: noOrders
    integer, intent(out), dimension(1 : noOrders) :: order
    integer i

    do i = 1, noOrders
        order(i) = intParameter("orderlist", i - 1)
    end do

end subroutine GetOrders         

!***************************
! PlotProgramVersionNo
!***************************
subroutine PlotProgramVersionNo
  
    call pgsvp(0.0, 1.0, 0.0, 1.0)
    call pgsch(0.9)
    call pgslw(2)
    call SetUpColourIndices
  
!    call pgsci(colourIndex(10))
    call pgsci(1)
    call pgmtxt("b", -0.02, 0.0, 0.0, "rgsimplot version 1.17")

end subroutine PlotProgramVersionNo


!**************************************************
! subroutine PlotBackgroundRegion
!**************************************************
subroutine PlotBackGroundRegion(fileName, dataSetH, instrument)
    
    character(len=*), intent(in) :: fileName
    type(DataSetT), intent(in) :: dataSetH
    character(len=*), intent(in) :: instrument
    
    character(len=15) :: string
    
    string = trim(instrument) // "_BACKGROUND"   


    call PlotFitsRegion(fileName, datasetH, string, 10, 2, "XDSP_CORR")

!    call pgsci(colourIndex(10))
    call pgsci(1) 
end subroutine PlotBackGroundRegion
!****************************************
! SetUpColourIndice
! Colour indices used to plot fits regions
!****************************************
subroutine SetUpColourIndices
    
    integer i
    
    do i = 1, 10
        colourIndex(i) = i + 1
    end do

    call pgscr(colourIndex(1),  0.0, 1.0, 1.0)                   ! Cyan
    call pgscr(colourIndex(2),  1.0, 0.0, 1.0)                   ! Magenta
    call pgscr(colourIndex(3),  1.0, 1.0, 0.0)                   ! Yellow
    call pgscr(colourIndex(4),  0.0, 128.0/255.0, 128.0/255.0)   ! Dark cyan
    call pgscr(colourIndex(5),  0.0, 0.0, 128.0/255.0)           ! Dark blue 
    call pgscr(colourIndex(6),  128.0/255.0, 128.0/255.0, 0.0)   ! Dark yellow
    call pgscr(colourIndex(7),  0.0, 128.0/255.0, 0.0)           ! Dark green 
    call pgscr(colourIndex(8),  128.0/255.0, 0.0, 128.0/255.0)   ! Dark magenta 
    call pgscr(colourIndex(9),  128.0/255.0, 0.0, 0.0)           ! Dark red
    
 !   if (display) then        ! White background
 !       call pgscr(colourIndex(10), 1.0, 1.0, 1.0)               ! White
 !   else 
 !       call pgscr(colourIndex(10), 0.0, 0.0, 0.0)               ! Black
 !   end if
    call pgscr(colourIndex(10), 1.0, 0.0, 0.0)                 ! Red for background mask
end subroutine SetUpColourIndices

!*******************************************************
! plotFitsRegion
!
!******************************************************
subroutine PlotBackgroundRegion1(datasetH)
  integer i, j, k, npoints, nregions
  real(kind=Single), pointer :: yval(:,:), xval(:,:)
  type(DataSetT), intent(in) :: dataSetH
  type(TableT) :: tableSetH
  type(ColumnT) :: xCol, yCol
 
   
  tableSetH = table(dataSetH, "RGS1_BACKGROUND")
   
  call pgqci(k)
    
  xCol = column(tableSetH, "X", read)
  xval => real32VectorData(xCol)
  yCol = column(tableSetH, "Y", read)
  yval => real32VectorData(yCol)

  npoints = ubound(xval, 1) - lbound(xval, 1) + 1
  nregions = ubound(xval, 2) - lbound(xval, 2) + 1
  !write(*, *) "No of points =", npoints, " no of regions=", nregions

  !*************************************************
  ! Draw the line
  ! Each order is represented by a different style line
  !*************************************************
  call pgsls(2)
!  call pgsci(colourIndex(10))
  call pgsci(1)
  call pgslw(1) 
  call pgmove(xval(0, 0), yval(0, 0))
 


  do i = 0, nregions - 1

      call pgmove(xval(0, i), yval(0, i))

      do j = 1, npoints - 1
          call pgdraw(xval(j, i), yval(j, i))
      end do
  end do
  
  

   call release(xCol)
   call release(yCol)

       
    
   call release(tableSetH)

end subroutine PlotBackgroundRegion1

!*******************************************************
! GetSourceDetails
! Gets the RA and dec for each source in the list.
!
!******************************************************
subroutine GetSourceDetails(datasetH, noSources, sourceNumber, &
                            sourceRA, sourceDec, sourceName)

    type(DataSetT), intent(in) :: dataSetH
    integer, intent(inout)     :: noSources
    integer, intent(out), dimension(1 : noSources)           :: sourceNumber
    real(kind=single), intent(out), dimension(1 : noSources) :: sourceRA
    real(kind=single), intent(out), dimension( 1: noSources) :: sourceDec
    character(len=*), intent(out), dimension(1 : noSources) :: sourceName

    real(kind=single), dimension(:), pointer :: ra, dec 
    
    type(tableT)  :: sourceTable
    type(columnT) :: sourceRAColumn
    type(columnT) :: sourceDecColumn
    type(columnT) :: sourceNameColumn


    character(len=100) :: string
    integer i, count, noRows, number
    

    sourceTable = table(dataSetH, "SRCLIST")   ! In earlier versions this was SOURCES

    noRows = numberofRows(sourceTable)

    !********************************************
    ! Get the RAs and decs of the sources
    !********************************************
    sourceRAColumn = column(sourceTable, "RA", read)    ! J2000 in decimal degrees
    ra => real32Data(sourceRAColumn) 

    sourceDecColumn = column(sourceTable, "DEC", read)  ! J2000 in decimal degrees
    dec => real32Data(sourceDecColumn) 

    sourceNameColumn = column(sourceTable, "LABEL", read)  ! J2000 in decimal degrees
    



    count = 0

    do i = 1, noSources
        number = intParameter("srcidlist", i - 1)
   
        if (number .gt. 0 .and. number .le. noRows) then
            count = count + 1
            sourceNumber(count) = number
   
            sourceRA(count)  = ra(number - 1)
            sourceDec(count) = dec(number - 1)
            sourceName(count) = trim(stringCell(sourceNameColumn, number - 1))
        else
            write(string, '(a, i5)') "number ", number
            call warning("Source doesn't exist in source table", trim(string))
        end if  
    end do

    noSources = count
    if (noSources .gt. 3) then
        write(string, '(a, i3)') "number=", noSources
        call warning("Unable to plot the coordinates of more than 3 sources", trim(string) )
    end if

    call release(sourceRAColumn)
    call release(sourceDecColumn)
    call release(sourceNameColumn)
    call release(sourceTable)

end subroutine GetSourceDetails
!***********************************************************************
! subroutine EffectiveRange 
! Finds the effective  range of an image fopr display purposes.
!***********************************************************************
subroutine EffectiveRange(image, qlo, qhi, effmin, effmax)

    real, intent(in)          :: image(:,:)  ! real image to examine
    real, intent(in)          :: qlo         ! quantile to ignore at low end
                                             ! e.g. 0.05 to ignore lowest 5%
    real, intent(in)          :: qhi         ! quantile to ignore at high end
    real, intent(out)         :: effmin      ! effective minimum value
    real, intent(out)         :: effmax      ! effective maximum value

    real :: amin, amax, scale
    integer, parameter :: nbins = 30000      ! fairly arbitrary histogram size
    integer, dimension(:), allocatable :: hist
    integer :: npts, ix, iy, j, n, nskip, nsum
    integer :: nx, ny, numNoneZero

    amin = minval(image)          ! seems to ignore null values
    amax = maxval(image) 

    !***************************************************
    ! Check for all image pixels values the same
    !***************************************************
    if (abs(amax - amin) .lt. 1e-6) then
        call warning("flat image", "All image pixel values have the same value")
        effmin = amin - 0.5
        effmax = amin + 0.5
        return
    end if
    
    if (abs(amax - amin) .lt. 1.1) then
        effmin = amin
        effmax = amax
        return
    end if

    nx = size(image, 1)
    ny = size(image, 2)

    numNoneZero = 0
    do iy = 1, ny
        do ix = 1, nx
            if(image(ix, iy) .gt. amin .and. &
               image(ix, iy) .lt. amax) then         ! this to ignore nulls
                if (image(ix, iy) .gt. 0.0) numNoneZero = numNoneZero + 1
            end if
        end do
    end do
    if ((real(numNoneZero) / real(nx * ny)) .lt. 0.10) then
        effmin = amin
        effmax = amax
        return
    end if


    allocate(hist(0 : nbins)) 

    hist = 0                      ! initialise histogram array
    npts = 0                      ! will count non-null values

    scale = real(nbins) / (amax - amin + 100.0 * epsilon(0.0))

    do iy = 1, ny
        do ix = 1, nx
            if(image(ix, iy) .gt. amin .and. &
               image(ix, iy) .lt. amax) then         ! this to ignore nulls
                npts = npts + 1
                n = (image(ix, iy) - amin) * scale
                hist(n) = hist(n) + 1
            end if
        end do
    end do

    !*************************************************************
    ! now find how far up histogram to go to exceed qlo proportion
    ! of the pixels.
    !************************************************************

    nsum  = 0
    nskip = nint(qlo * npts)
    do j = 0, nbins
        nsum = nsum + hist(j)
       if(nsum .gt. nskip) then    ! reached threshold, set level
           effmin = amin + j / scale
           EXIT
       end if
    end do

    nsum = 0
    nskip = nint(qhi * npts)

    do j = nbins, 0, -1
        nsum = nsum + hist(j)
        if(nsum .gt. nskip) then
            effmax = amin + j / scale
            EXIT
        end if
    end do   

    !*****************************************************
    ! fudge here to stop PGPLOT problem if all values same.
    !******************************************************

    if(effmax .eq. effmin) effmax = effmin + 100.0 * epsilon(0.0)
  
    deallocate(hist)

end subroutine EffectiveRange
!************************************************************
! Subroutine SortPoints
! Sorts  array of type POINTS into increasing POINT.x order
!************************************************************
subroutine SortPoints(size, array)
    integer, intent(in) :: size
    type(POINT), intent(inout), dimension(1 : size) :: array
    integer(kind = int32) :: i, ir, j, l
    type(POINT) :: rra


    if (size .lt. 2) return

    l = size / 2 + 1
    ir = size   
    do 
        if (l .gt. 1) then
            l = l - 1
            rra = array(l)    
        else
            rra = array(ir)
             
            array(ir) = array(1)
            ir = ir - 1
            if (ir .le. 1) then
                array(1) = rra
                exit
            end if
        end if

        i = l
        j = l + l
        do while(j .le. ir)
       
            if (j .lt. ir) then
                if(array(j)%x .lt. array(j + 1)%x) j = j + 1
            end if
            if (rra%x .lt. array(j)%x) then
                array(i) = array(j)
      
                i = j
                j = j + j
            else
                exit
            end if
            
        end do   
        array(i) = rra    
    end do
 
end subroutine SortPoints
end module rgsimplot_mod

























